
# TODO: replace with direct calls to pkg_resources
from weblogo.utils import resource_string, resource_stream, resource_filename


def data_string(name):
    return resource_string(__name__, 'data/' + name, __file__)


def data_stream(name):
    return resource_stream(__name__, 'data/' + name, __file__)


def data_filename(name):
    return resource_filename(__name__, 'data/' + name, __file__)



#!/usr/bin/env python

#  Copyright (c) 2006, The Regents of the University of California, through
#  Lawrence Berkeley National Laboratory (subject to receipt of any required
#  approvals from the U.S. Dept. of Energy).  All rights reserved.

#  This software is distributed under the new BSD Open Source License.
#  <http://www.opensource.org/licenses/bsd-license.html>
#  Please see the LICENSE.txt file that should have been included
#  as part of this package.


""" A tool for converting multiple sequence alignments from
one format to another.
"""

import sys
from optparse import OptionGroup

from weblogo import seq_io
from weblogo.seq import SeqList, Seq, nucleic_alphabet
from weblogo.utils.deoptparse import DeOptionParser
from weblogo.transform import mask_low_complexity

__version__ = "1.0.0"
description = """ A tool for converting multiple sequence alignments from
one format to another. """


def main():
    # ------ Parse Command line ------
    parser = _build_option_parser()
    (opts, args) = parser.parse_args(sys.argv[1:])
    if args:
        parser.error("Unparsable arguments: %s " % args)

    seqs = opts.reader.read(opts.fin)

    if opts.trans_seg:
        seqs = SeqList([mask_low_complexity(s) for s in seqs])

    if opts.subsample is not None:
        from random import random
        frac = opts.subsample
        ss = []
        for s in seqs:
            if random() < frac:
                ss.append(s)
        seqs = SeqList(ss)

    if opts.reverse:
        seqs = SeqList([s.reverse() for s in seqs])

    if opts.complement:
        seqs = SeqList([Seq(s, alphabet=nucleic_alphabet) for s in seqs])
        seqs = SeqList([s.complement() for s in seqs])

    opts.writer.write(opts.fout, seqs)


def _build_option_parser():
    parser = DeOptionParser(usage="%prog [options]  < sequence_data.fa > sequence_logo.eps",
                            description=description,
                            version=__version__,
                            add_verbose_options=False
                            )

    io_grp = OptionGroup(parser, "Input/Output Options", )
    parser.add_option_group(io_grp)

    io_grp.add_option("-f", "--fin",
                      dest="fin",
                      action="store",
                      type="file_in",
                      default=sys.stdin,
                      help="Sequence input file (default: stdin)",
                      metavar="FILENAME")

    io_grp.add_option("", "--format-in",
                      dest="reader",
                      action="store", type="dict",
                      default=seq_io,
                      choices=seq_io.format_names(),
                      help="Multiple sequence alignment format: (%s)" %
                           ', '.join([f.names[0] for f in seq_io.formats]),
                      metavar="FORMAT")

    io_grp.add_option("-o", "--fout", dest="fout",
                      type="file_out",
                      default=sys.stdout,
                      help="Output file (default: stdout)",
                      metavar="FILENAME")

    trans_grp = OptionGroup(parser, "Transformations", )
    parser.add_option_group(trans_grp)

    trans_grp.add_option("", "--seg",
                         dest="trans_seg",
                         action="store_true",
                         default=False,
                         help="Mask low complexity regions in protein sequences.",
                         metavar="TRUE/FALSE")

    trans_grp.add_option("", "--subsample",
                         dest="subsample",
                         action="store",
                         type="float",
                         default=None,
                         help="Return a random subsample of the sequences.",
                         metavar="FRACTION")

    trans_grp.add_option("", "--reverse",
                         dest="reverse",
                         action="store_true",
                         default=False,
                         help="reverse sequences",
                         metavar="TRUE/FALSE")

    trans_grp.add_option("", "--complement",
                         dest="complement",
                         action="store_true",
                         default=False,
                         help="complement DNA sequences",
                         metavar="TRUE/FALSE")

    # Writers
    out_formats = []
    for f in seq_io.formats:
        if hasattr(f, "write"):
            out_formats.append(f)
    out_choices = {}
    for f in out_formats:
        out_choices[f.names[0]] = f
    out_names = [f.names[0] for f in out_formats]

    io_grp.add_option("-F", "--format-out",
                      dest="writer",
                      action="store", type="dict",
                      default=seq_io.fasta_io,
                      choices=out_choices,
                      help="Multiple sequence alignment output format: (%s) (Default: fasta)" %
                           ', '.join(out_names),
                      metavar="FORMAT")

    return parser



"""
Logo formatting. Each formatter is a function f(data, format) that draws
a representation of the logo. The main graphical formatter is eps_formatter. A mapping
'formatters' containing all available formatters . Each formatter returns binary data. The eps
and data formats can decoded to strings, e.g. eps_as_string = eps_data.decode()
"""

import shutil
from subprocess import Popen, PIPE
from math import log
from string import Template
import os

from .utils import resource_string
from .logo import LogoData, LogoFormat
from .color import Color

__all__ = ['pdf_formatter', 'jpeg_formatter', 'svg_formatter', 'png_formatter',
           'png_print_formatter', 'txt_formatter', 'eps_formatter', 'formatters',
           'default_formatter', 'GhostscriptAPI']

std_units = {
    "bits": 1. / log(2),
    "nats": 1.,
    "digits": 1. / log(10),
    "kT": 1.,
    "kJ/mol": 8.314472 * 298.15 / 1000.,
    "kcal/mol": 1.987 * 298.15 / 1000.,
    "probability": None,
}
"""Some text"""


def pdf_formatter(logodata: LogoData, logoformat: LogoFormat) -> bytes:
    """ Generate a logo in PDF format."""
    eps = eps_formatter(logodata, logoformat).decode()
    gs = GhostscriptAPI()
    return gs.convert('pdf', eps, logoformat.logo_width, logoformat.logo_height)


def _bitmap_formatter(logodata: LogoData, logoformat: LogoFormat, device: str) -> bytes:
    eps = eps_formatter(logodata, logoformat).decode()
    gs = GhostscriptAPI()
    return gs.convert(device, eps,
                      logoformat.logo_width, logoformat.logo_height, logoformat.resolution)


def jpeg_formatter(logodata: LogoData, logoformat: LogoFormat) -> bytes:
    """ Generate a logo in JPEG format."""
    return _bitmap_formatter(logodata, logoformat, device="jpeg")


def svg_formatter(logodata: LogoData, logoformat: LogoFormat) -> bytes:
    """ Generate a logo in Scalable Vector Graphics (SVG) format.
    Requires the program 'pdf2svg' be installed.
    """
    pdf = pdf_formatter(logodata, logoformat)

    command = shutil.which('pdf2svg')
    if command is None:
        raise EnvironmentError("Scalable Vector Graphics (SVG) format requires the program"
                               "'pdf2svg'.")  # pragma: no cover

    import tempfile
    fpdfi, fname_pdf = tempfile.mkstemp(suffix=".pdf")
    fsvgi, fname_svg = tempfile.mkstemp(suffix=".svg")
    try:

        fpdf2 = open(fname_pdf, 'w')
        fpdf2.buffer.write(pdf)
        fpdf2.seek(0)

        args = [command, fname_pdf, fname_svg]
        p = Popen(args)
        (out, err) = p.communicate()

        fsvg = open(fname_svg)
        return fsvg.read().encode()
    finally:
        os.remove(fname_svg)
        os.remove(fname_pdf)


def png_formatter(logodata: LogoData, logoformat: LogoFormat) -> bytes:
    """ Generate a logo in PNG format."""
    return _bitmap_formatter(logodata, logoformat, device="png")


def png_print_formatter(logodata: LogoData, logoformat: LogoFormat) -> bytes:
    """ Generate a logo in PNG format with print quality (600 DPI) resolution."""
    logoformat.resolution = 600
    return _bitmap_formatter(logodata, logoformat, device="png")


def txt_formatter(logodata: LogoData, logoformat: LogoFormat) -> bytes:
    """ Create a text representation of the logo data.
    """
    return str(logodata).encode()


def eps_formatter(logodata: LogoData, logoformat: LogoFormat) -> bytes:
    """ Generate a logo in Encapsulated Postscript (EPS)"""
    substitutions = {}
    from_format = [
        "creation_date", "logo_width", "logo_height",
        "lines_per_logo", "line_width", "line_height",
        "line_margin_right", "line_margin_left", "line_margin_bottom",
        "line_margin_top", "title_height", "xaxis_label_height",
        "creator_text", "logo_title", "logo_margin",
        "stroke_width", "tic_length",
        "stacks_per_line", "stack_margin",
        "yaxis_label", "yaxis_tic_interval", "yaxis_minor_tic_interval",
        "xaxis_label", "xaxis_tic_interval", "number_interval",
        "fineprint", "shrink_fraction", "errorbar_fraction",
        "errorbar_width_fraction",
        "errorbar_gray", "small_fontsize", "fontsize",
        "title_fontsize", "number_fontsize", "text_font",
        "logo_font", "title_font",
        "logo_label", "yaxis_scale", "end_type",
        "debug", "show_title", "show_xaxis",
        "show_xaxis_label", "show_yaxis", "show_yaxis_label",
        "show_boxes", "show_errorbars", "show_fineprint",
        "rotate_numbers", "show_ends", "stack_height",
        "stack_width"
    ]

    for sf in from_format:
        substitutions[sf] = getattr(logoformat, sf)

    substitutions["shrink"] = str(logoformat.show_boxes).lower()

    def format_color(color: Color) -> str:    # (no fold)
        return " ".join(("[", str(color.red), str(color.green),
                         str(color.blue), "]"))

    substitutions["default_color"] = format_color(logoformat.default_color)

    data = []

    # Unit conversion. 'None' for probability units
    conv_factor = std_units[logoformat.unit_name]

    data.append("StartLine")

    seq_from = logoformat.logo_start - logoformat.first_index
    seq_to = logoformat.logo_end - logoformat.first_index + 1

    # seq_index : zero based index into sequence data
    # logo_index : User visible coordinate, first_index based
    # stack_index : zero based index of visible stacks
    for seq_index in range(seq_from, seq_to):
        # logo_index = seq_index + logoformat.first_index
        stack_index = seq_index - seq_from

        if stack_index != 0 and (stack_index % logoformat.stacks_per_line) == 0:
            data.append("")
            data.append("EndLine")
            data.append("StartLine")
            data.append("")

        data.append("(%s) StartStack" % logoformat.annotate[seq_index])

        if conv_factor:
            stack_height = logodata.entropy[seq_index] * std_units[logoformat.unit_name]
        else:
            stack_height = 1.0  # probability   # pragma: no cover

        # Sort by frequency. If equal frequency then reverse alphabetic
        # (So sort reverse alphabetic first, then frequencty)
        # TODO: doublecheck this actual works
        s = list(zip(logodata.counts[seq_index], logodata.alphabet))
        s.sort(key=lambda x: x[1])
        s.reverse()
        s.sort(key=lambda x: x[0])

        if not logoformat.reverse_stacks:
            s.reverse()         # pragma: no cover

        C = float(sum(logodata.counts[seq_index]))
        if C > 0.0:
            fraction_width = 1.0
            if logoformat.scale_width:
                fraction_width = logodata.weight[seq_index]
                # print(fraction_width, file=sys.stderr)
            for rank, c in enumerate(s):
                color = logoformat.color_scheme.symbol_color(seq_index, c[1], rank)
                data.append(" %f %f %s (%s) ShowSymbol" % (
                    fraction_width, c[0] * stack_height / C,
                    format_color(color), c[1]))

        # Draw error bar on top of logo. Replaced by DrawErrorbarFirst above.
        if logodata.entropy_interval is not None and conv_factor and C > 0.0:

            low, high = logodata.entropy_interval[seq_index]
            center = logodata.entropy[seq_index]
            low *= conv_factor
            high *= conv_factor
            center *= conv_factor
            if high > logoformat.yaxis_scale:
                high = logoformat.yaxis_scale       # pragma: no cover

            down = (center - low)
            up = (high - center)
            data.append(" %f %f DrawErrorbar" % (down, up))

        data.append("EndStack")
        data.append("")

    data.append("EndLine")
    substitutions["logo_data"] = "\n".join(data)

    # Create and output logo
    template = resource_string(__name__, 'template.eps', __file__).decode()
    logo = Template(template).substitute(substitutions)

    return logo.encode()


formatters = {
    'eps': eps_formatter,
    'pdf': pdf_formatter,
    'png': png_formatter,
    'png_print': png_print_formatter,
    'jpeg': jpeg_formatter,
    'svg': svg_formatter,
    'logodata': txt_formatter,
}
"""Map between output format names and corresponing logo formatter"""


default_formatter = eps_formatter
"""The default logo formatter."""


class GhostscriptAPI(object):
    """Interface to the command line program Ghostscript ('gs')"""

    formats = ('png', 'pdf', 'jpeg')

    def __init__(self, path: os.PathLike = None) -> None:
        """
        Raises:
            EnvironmentError: If cannot find Ghostscript executable on
                path
        """
        command = shutil.which('gs', path=path)
        if command is None:
            command = shutil.which('gswin32c.exe', path=path)   # pragma: no cover
        if command is None:
            raise EnvironmentError("Could not find Ghostscript on path. "
                                   "There should be either a gs executable or a gswin32c.exe on "
                                   "your system's path")    # pragma: no cover

        self.command = command

    def version(self) -> str:
        """Returms: The ghostscript version string"""
        args = [self.command, '--version']
        try:
            p = Popen(args, stdout=PIPE)
            (out, err) = p.communicate()
        except OSError:     # pragma: no cover
            raise RuntimeError("Cannot communicate with ghostscript.")  # pragma: no cover
        return out.strip()

    def convert(self,
                format: str,
                postscript: str,
                width: int,
                height: int,
                resolution: int = 300) -> bytes:
        """Convert a string of postscript into a different graphical format

        Supported foramts are 'png', 'pdf', and 'jpeg'.

        Raises:
            ValueError: For an unregonized format.
        """
        device_map = {'png': 'png16m', 'pdf': 'pdfwrite', 'jpeg': 'jpeg'}

        try:
            device = device_map[format]
        except KeyError:                                # pragma: no cover
            raise ValueError("Unsupported format.")

        args = [self.command,
                "-sDEVICE=%s" % device,
                "-dPDFSETTINGS=/printer",
                # "-q",   # Quite: Do not dump messages to stdout.
                "-sstdout=%stderr",  # Redirect messages and errors to stderr
                # fix issue 36, problems with ghostscript 9.10
                "-dColorConversionStrategy=/LeaveColorUnchanged",
                "-sOutputFile=-",  # Stdout
                "-dDEVICEWIDTHPOINTS=%s" % str(width),
                "-dDEVICEHEIGHTPOINTS=%s" % str(height),
                "-dSAFER",  # For added security
                "-dNOPAUSE", ]

        if device != 'pdf':
            args.append("-r%s" % str(resolution))
            if resolution < 300:  # Antialias if resolution is Less than 300 DPI
                args.append("-dGraphicsAlphaBits=4")
                args.append("-dTextAlphaBits=4")
                args.append("-dAlignToPixels=0")

        args.append("-")  # Read from stdin. Must be last argument.

        error_msg = "Unrecoverable error : Ghostscript conversion failed " \
                    "(Invalid postscript?). %s" % " ".join(args)

        try:
            p = Popen(args, stdin=PIPE, stdout=PIPE, stderr=PIPE)
            (out, err) = p.communicate(postscript.encode())
        except OSError:                         # pragma: no cover
            raise RuntimeError(error_msg)       # pragma: no cover

        if p.returncode != 0:                   # pragma: no cover
            error_msg += '\nReturn code: %i\n' % p.returncode
            if err is not None:
                error_msg += err
            raise RuntimeError(error_msg)

        return out

# end class Ghostscript



#!/usr/bin/env python

#  Copyright (c) 2006, The Regents of the University of California, through
#  Lawrence Berkeley National Laboratory (subject to receipt of any required
#  approvals from the U.S. Dept. of Energy).  All rights reserved.

#  This software is distributed under the new BSD Open Source License.
#  <http://www.opensource.org/licenses/bsd-license.html>
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  (1) Redistributions of source code must retain the above copyright notice,
#  this list of conditions and the following disclaimer.
#
#  (2) Redistributions in binary form must reproduce the above copyright
#  notice, this list of conditions and the following disclaimer in the
#  documentation and or other materials provided with the distribution.
#
#  (3) Neither the name of the University of California, Lawrence Berkeley
#  National Laboratory, U.S. Dept. of Energy nor the names of its contributors
#  may be used to endorse or promote products derived from this software
#  without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

import re
import unittest
from io import StringIO

from weblogo.utils import (isfloat, isint, isblank, remove_whitespace, Reiterate, crc32, crc64,
                           Token, group_count, ArgumentError, FileIndex, invert_dict,
                           resource_filename, resource_string, resource_stream)


class test_utils(unittest.TestCase):
    def test_isfloat(self):
        self.assertTrue(isfloat('0.5'))
        self.assertTrue(isfloat(' 0'))
        self.assertTrue(isfloat('+1000000000  '))
        self.assertTrue(isfloat('2'))
        self.assertTrue(isfloat('0000.2323'))
        self.assertTrue(isfloat('0.1e-23'))
        self.assertTrue(isfloat(' -0.5e+23'))
        self.assertFalse(isfloat(None))
        self.assertFalse(isfloat(''))
        self.assertFalse(isfloat('asdad'))
        self.assertFalse(isfloat('q34sd'))
        self.assertFalse(isfloat('92384.kjdfghiksw'))
        self.assertFalse(isfloat('adf!@#nn'))

    def test_isint(self):
        self.assertTrue(isint('0'))
        self.assertTrue(isint('-1'))
        self.assertTrue(isint('10'))
        self.assertTrue(isint('100101012234'))
        self.assertTrue(isint('000'))
        self.assertFalse(isint(None))
        self.assertFalse(isint(''))
        self.assertFalse(isint('asdad'))
        self.assertFalse(isint('q34sd'))
        self.assertFalse(isint('0.23'))
        self.assertFalse(isint('adf!@#nn'))

    def test_remove_whitespace(self):
        self.assertEqual(remove_whitespace("  kjashd askjdh askjdh\tasdf"),
                         "kjashdaskjdhaskjdhasdf")

    def test_isblank(self):
        blank = ('', ' ', '\n', '\t \n\n')
        not_blank = (' a',)
        for s in blank:
            self.assertTrue(isblank(s))
        for s in not_blank:
            self.assertFalse(isblank(s))

        self.assertFalse(isblank(123))

    def test_group_count(self):
        test = 'aaabbbbcccddea'
        out = group_count(test)
        self.assertEqual(tuple(out),
                         (('a', 3), ('b', 4), ('c', 3), ('d', 2), ('e', 1), ('a', 1)))

    def test_reiterate(self):
        i = Reiterate(iter("123456"))
        for item in i:
            pass
        self.assertRaises(StopIteration, i.next)
        self.assertFalse(i.has_item())
        self.assertTrue(i.peek() is None)

        # pushback
        i = Reiterate(iter("123456"))
        next(i)
        i.push("0")
        self.assertEqual("0", next(i))
        p = i.peek()
        n = next(i)
        self.assertEqual(p, n)
        self.assertEqual(i.index(), 2)
        self.assertTrue(i.has_item())

        # Repeated application of Reiterate should return same iterator.
        self.assertTrue(i is iter(i))
        self.assertTrue(i is Reiterate(i))

    def test_token(self):
        t = Token('kind', 'some data', 4, 3)
        str(t)
        r = repr(t)
        t2 = eval(r)
        self.assertEqual(t2.typeof, 'kind')

    def test_invert_dict(self):
        d = dict(a=3, b=4)
        invd = invert_dict(d)
        self.assertTrue(3 in invd)
        self.assertEqual(invd[3], 'a')

    def test_crc64(self):
        self.assertEqual(crc64("IHATEMATH"), "E3DCADD69B01ADD1")

    def test_crc32(self):
        self.assertEqual(crc32("Test the CRC-32 of this string."),
                         "%08X" % 1571220330)

    def test_ArgumentValueError(self):
        message = "Some message"
        component = "whatsit"
        try:
            raise ArgumentError(message, component)
        except ArgumentError as err:
            self.assertEqual(err.msg, message)
            self.assertEqual(err.key, component)
        try:
            raise ArgumentError(message, component, 10)
        except ValueError as err:
            self.assertEqual(err.msg, message)
            self.assertEqual(err.key, component)
            self.assertEqual(err.value, 10)

    def test_file_index(self):
        stream = StringIO(tfile)
        idx = FileIndex(stream)
        self.assertTrue(idx[0].startswith('line 0'))
        self.assertTrue(idx[4].startswith('line 4'))

        def parser(f):
            return int(f.readline().split()[1])

        idx = FileIndex(stream, parser=parser)
        self.assertEqual(len(idx), 5)
        self.assertEqual(idx[0], 0)
        self.assertEqual(idx[4], 4)

        key = re.compile(r"(line \d)")

        def linekey(line):
            k = key.search(line)
            if k is None:
                return None
            return k.group(1)

        idx = FileIndex(stream, linekey=linekey, parser=parser)
        self.assertEqual(len(idx), 4)
        self.assertEqual(idx[0], 0)
        self.assertEqual(idx[3], 4)
        self.assertRaises(IndexError, idx.__getitem__, 5)

        # print idx._key_dict
        self.assertEqual(idx['line 1'], 1)
        self.assertEqual(idx['line 4'], 4)
        self.assertTrue('line 1' in idx)
        self.assertFalse('Blah' in idx)
        self.assertFalse(20 in idx)

        # Test iteration over values
        t = 0
        for v in idx:
            t += v
        self.assertEqual(t, 8)

    def test_resource(self):
        fn = resource_filename(__name__, 'data/cap.fa', __file__)
        self.assertTrue(fn.endswith('data/cap.fa'))
        f = resource_stream(__name__, 'data/cap.fa', __file__)
        f.close()
        s = resource_string(__name__, 'data/cap.fa', __file__).decode()
        self.assertTrue(s.startswith('>aldB'))


tfile = """line 0
line 1
Blah
line 3
line 4
"""

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python


"""Read GenBank flat files.

Currently only reads sequence data and not annotations.

"""

from ..seq import Alphabet, Seq, SeqList
from ..utils import isblank

names = ('genbank',)
extensions = ('gb', 'genbank', 'gbk')


def read(fin, alphabet=None):
    """Read and parse a file of genbank records.

    Args:
    fin -- A stream or file to read
    alphabet -- The expected alphabet of the data, if given

    Returns:
    SeqList -- A list of sequences

    Raises:
    ValueError -- If the file is unparsable
    """
    seqs = [s for s in iterseq(fin, alphabet)]
    return SeqList(seqs)


def iterseq(fin, alphabet=None):
    """ Iterate over genbank records

    Args:
    fin -- A stream or file to read
    alphabet -- The expected alphabet of the data, if given

    Yeilds:
    Seq -- One alphabetic sequence at a time.

    Raises:
    ValueError -- If the file is unparsable
    """
    alphabet = Alphabet(alphabet)

    header, block, data = range(3)
    state = header
    seq = []
    for L, line in enumerate(fin):
        if isblank(line):
            continue
        if state == header:
            if not line.startswith('LOCUS'):
                raise ValueError(
                   "Cannot find start of record at line %d" % L)
            state = block
        elif state == block:
            if line.startswith('ORIGIN') or line.startswith('//'):
                state = data
        elif state == data:
            if line.startswith('//'):
                yield Seq(''.join(seq), alphabet)
                seq = []
                state = block
            else:
                seq.extend(line.split()[1:])



#!/usr/bin/env python

#  Copyright (c) 2006, The Regents of the University of California, through
#  Lawrence Berkeley National Laboratory (subject to receipt of any required
#  approvals from the U.S. Dept. of Energy).  All rights reserved.

#  This software is distributed under the new BSD Open Source License.
#  <http://www.opensource.org/licenses/bsd-license.html>
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  (1) Redistributions of source code must retain the above copyright notice,
#  this list of conditions and the following disclaimer.
#
#  (2) Redistributions in binary form must reproduce the above copyright
#  notice, this list of conditions and the following disclaimer in the
#  documentation and or other materials provided with the distribution.
#
#  (3) Neither the name of the University of California, Lawrence Berkeley
#  National Laboratory, U.S. Dept. of Energy nor the names of its contributors
#  may be used to endorse or promote products derived from this software
#  without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#  POSSIBILITY OF SUCH DAMAGE.

from io import StringIO

import unittest

from weblogo.seq import protein_alphabet, dna_alphabet
from weblogo.seq_io import nbrf_io, clustal_io, plain_io
from . import data_stream


class test_nbrf_io(unittest.TestCase):
    def test_parse_cox2(self):
        f = data_stream('cox2.nbrf')
        seqs = nbrf_io.read(f)
        self.assertEqual(len(seqs), 5)
        self.assertEqual(len(seqs[1]), 210)
        self.assertEqual(str(seqs[0]),
                         "MAFILSFWMIFLLDSVIVLLSFVCFVCVWICALLFSTVLLVSKLNNIYCTWDFTASKFIDVYWFTIGGMFSLG"
                         "LLLRLCLLLYFGHLNFVSFDLCKVVGFQWYWVYFIFGETTIFSNLILESDYMIGDLRLLQCNHVLTLLSLVIY"
                         "KLWLSAVDVIHSFAISSLGVKVENLVAVMK")
        self.assertEqual(seqs[0].alphabet, protein_alphabet)
        f.close()

    def test_parse_crab(self):
        f = data_stream('crab.nbrf')
        seqs = nbrf_io.read(f)
        self.assertEqual(seqs[0].alphabet, protein_alphabet)
        self.assertEqual(len(seqs), 9)
        self.assertEqual(seqs[2].name, "CRAB_CHICK")
        self.assertEqual(seqs[2].description,
                         "ALPHA CRYSTALLIN B CHAIN (ALPHA(B)-CRYSTALLIN).")
        f.close()

    def test_parse_dna(self):
        f = data_stream('dna.pir')
        seqs = nbrf_io.read(f)
        self.assertEqual(seqs[0].alphabet, dna_alphabet)
        self.assertEqual(len(seqs), 10)
        f.close()

    def test_parse_examples(self):
        f = data_stream('rhod.pir')
        seqs = nbrf_io.read(f)
        self.assertEqual(seqs[0].alphabet, protein_alphabet)
        self.assertEqual(len(seqs), 3)
        f.close()

    def test_parse_protein(self):
        f = data_stream('protein.pir')
        seqs = nbrf_io.read(f)
        self.assertEqual(seqs[0].alphabet, protein_alphabet)
        self.assertEqual(len(seqs), 10)
        f.close()

    def test_parse_clustal_fail(self):
        # should fail with parse error
        f = StringIO(clustal_io.example)
        self.assertRaises(ValueError,
                          nbrf_io.read, f, protein_alphabet)

    def test_parse_plain_fail(self):
        # should fail with parse error
        f = StringIO(plain_io.example)
        self.assertRaises(ValueError,
                          nbrf_io.read, f)

    def test_pir_file_from_clustal(self):
        f = data_stream('clustalw.pir')
        seqs = nbrf_io.read(f)
        self.assertEqual(len(seqs), 2)
        self.assertEqual(seqs[1].endswith(
                'C-AATC-G-CAATG-G--CTTGAACCGGGTAAAAGTCGT-A----------------------------------------'
                '-----------------------------------------'),
                True)
        f.close()

    def test_parse_examples_alphabet(self):
        f = data_stream('rhod.pir')
        seqs = nbrf_io.read(f, alphabet=protein_alphabet)
        self.assertEqual(seqs[0].alphabet, protein_alphabet)
        self.assertEqual(len(seqs), 3)
        f.close()


if __name__ == '__main__':
    unittest.main()




from pkg_resources import get_distribution, DistributionNotFound
try:
    __version__ = get_distribution(__name__).version
except DistributionNotFound:
    __version__ = '?.?.?'


from .logo import *								# noqa: F401, F403
from .seq import *								# noqa: F401, F403
from .seq_io import *							# noqa: F401, F403
from .logo_formatter import *					# noqa: F401, F403




import shutil
from subprocess import Popen, PIPE

import pytest

# import weblogo

from . import data_stream


def _exec(args, outputtext, returncode=0, stdin=None):
    if not stdin:
        stdin = data_stream("cap.fa")
    args = ["weblogo"] + args
    p = Popen(args, stdin=stdin, stdout=PIPE, stderr=PIPE)
    (out, err) = p.communicate()
    if returncode == 0 and p.returncode > 0:
        print(err)
    assert returncode == p.returncode
    if returncode == 0:
        assert len(err) == 0

    if outputtext:
        out = out.decode()
        for item in outputtext:
            assert item in out

    stdin.close()


def test_malformed_options():
    _exec(["--notarealoption"], [], 2)
    _exec(["extrajunk"], [], 2)
    _exec(["-I"], [], 2)


def test_help_option():
    _exec(["-h"], ["options"])
    _exec(["--help"], ["options"])


# def test_version_option():
#     _exec(['--version'], weblogo.__version__[0:5])


def test_default_build():
    _exec([], ["%%Title:        Sequence Logo:"])


# Format options
def test_width():
    _exec(['-W', '1234'], ["/stack_width         1234"])
    _exec(['--stack-width', '1234'], ["/stack_width         1234"])


def test_height():
    _exec(['-W', '1000'], ["/stack_height        5000"])
    _exec(['-W', '1000', '--aspect-ratio', '2'], ["/stack_height        2000"])


def test_stacks_per_line():
    _exec(['-n', '7'], ["/stacks_per_line     7 def"])
    _exec(['--stacks-per-line', '7'], ["/stacks_per_line     7 def"])


def test_title():
    _exec(['-t', '3456'], ['/logo_title         (3456) def', '/show_title         True def'])
    _exec(['-t', ''], ['/logo_title         () def', '/show_title         False def'])
    _exec(['--title', '3456'], ['/logo_title         (3456) def', '/show_title         True def'])


def test_annotate():
    _exec(["--annotate", "1,2,3,4"], [], 2)
    _exec(["--annotate", "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,,"], [])


def test_color():
    _exec(['--color', 'black', 'AG', 'Purine'], [])
    _exec(['--color', 'not_a_color', 'AG', 'Purine'], [], 2)


def test_reverse_complement():
    _exec(['--complement'], [])
    _exec(['--reverse'], [])
    _exec(['--revcomp'], [])


def test_formats():
    _exec(['--format', 'eps'], [])
    _exec(['--format', 'png'], [])
    _exec(['--format', 'png_print'], [])
    _exec(['--format', 'pdf'], [])
    _exec(['--format', 'jpeg'], [])

    _exec(['--format', 'logodata'], [])


@pytest.mark.skipif(shutil.which('pdf2svg') is None,
                    reason="requires pdf2svg")
def test_formats_svg():
    _exec(['--format', 'svg'], [])



#!/usr/bin/env python

from setuptools import setup


long_description = """
WebLogo (https://github.com/WebLogo/weblogo) is a tool for creating sequence
logos from biological sequence alignments.  It can be run on the command line,
as a standalone webserver, as a CGI webapp, or as a python library.

The main WebLogo webserver is located at http://weblogo.threeplusone.com

Please consult the manual for installation instructions and more information:
./weblogo/htdocs/manual.html

(Also located at http://weblogo.threeplusone.com/manual.html.)
"""

setup(
    name="weblogo",
    python_requires='>=3.6',

    install_requires=['numpy', 'scipy', 'setuptools', 'setuptools_scm'],

    use_scm_version=True,
    setup_requires=['setuptools_scm'],


    description="WebLogo3 : Sequence Logos Redrawn",
    long_description=long_description,
    license='BSD',
    maintainer="Gavin Crooks",
    maintainer_email="gec@threeplusone.com",
    url="https://github.com/WebLogo/weblogo",

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Programming Language :: Python',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',

    ],

    packages=[
        'weblogo',
        'weblogo.seq_io',
        'weblogo.seq_io._nexus',
        'weblogo.utils',
    ],

    package_data={
        'weblogo': ['htdocs/*.*', 'htdocs/img/*.*', 'htdocs/examples/*.*',
                    'template.eps', 'data/*.*']
    },



    entry_points={
        'console_scripts': [
            'weblogo = weblogo._cli:main',
            'transformseq = weblogo._transformseq:main'
        ],
    },
)
